package com.cweb.controller;

import com.alibaba.fastjson.JSONObject;
import com.cweb.config.TuanYouConfig;
import com.github.pagehelper.PageInfo;
import com.hai.common.exception.ErrorCode;
import com.hai.common.exception.ErrorHelp;
import com.hai.common.exception.SysCode;
import com.hai.common.utils.CoordCommonUtil;
import com.hai.common.utils.PageUtil;
import com.hai.common.utils.ResponseMsgUtil;
import com.hai.model.ResponseData;
import com.hai.service.HighGasOilPriceService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

@Controller
@RequestMapping(value = "/highGas")
@Api(value = "团油业务接口")
public class HighGasController {

    private static Logger log = LoggerFactory.getLogger(HighGasController.class);

    @Resource
    private HighGasOilPriceService highGasOilPriceService;
    @RequestMapping(value="/getGasStoreList",method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "查询加油站列表")
    public ResponseData getGasStoreList(@RequestParam(name = "storeName", required = false) String storeName,
                                        @RequestParam(name = "distance", required = true) Integer distanceRecent,
                                        @RequestParam(name = "regionId", required = true) Long regionId,
                                        @RequestParam(name = "oilNoName", required = true) String oilNoName,
                                        @RequestParam(name = "longitude", required = true) String longitude,
                                        @RequestParam(name = "latitude", required = true) String latitude,
                                        @RequestParam(name = "pageNum", required = true) Integer pageNum,
                                        @RequestParam(name = "pageSize", required = true) Integer pageSize
    ) {
        try {
            if (StringUtils.isBlank(storeName)) {
                storeName = null;
            }
            List<Map<String, Object>> storeList = highGasOilPriceService.getStoreListByOilNo(storeName,regionId, oilNoName);
            for (Map<String, Object> store : storeList) {
                double distance = CoordCommonUtil.getDistance(Double.valueOf(store.get("latitude").toString()), Double.valueOf(store.get("longitude").toString()), Double.valueOf(latitude), Double.valueOf(longitude));
                store.put("distance", Math.round(distance/100d)/10d);
            }
            List<Map<String, Object>> distance = storeList.stream().sorted(Comparator.comparingDouble(entry ->  Double.valueOf(entry.get("distance").toString()))).collect(Collectors.toList());
            Iterator<Map<String, Object>> iterator = distance.iterator();
            while (iterator.hasNext()) {
                if ((int)Math.round(Double.valueOf(iterator.next().get("distance").toString()))  > distanceRecent.intValue()) {
                    iterator.remove();
                }
            }
            return ResponseMsgUtil.success(PageUtil.initPageInfoObj(pageNum,distance.size(),pageSize,new PageInfo<>(distance)));

        } catch (Exception e) {
            log.error("HighGasController -> getGasDetailByStoreKey() error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value="/getGasDetailByStoreKey",method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "根据门店key 查询")
    public ResponseData getGasDetailByStoreKey(@RequestParam(name = "storeKey", required = true) String storeKey) {
        try {

            JSONObject jsonObject = TuanYouConfig.queryGasInfoByGasId(storeKey);
            if (jsonObject != null && jsonObject.getString("code").equals("200")) {
                return ResponseMsgUtil.success(jsonObject.get("result"));
            }
            throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "未获取到加油站信息");

        } catch (Exception e) {
            log.error("HighGasController -> getGasDetailByStoreKey() error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value="/getGasPriceDetail",method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "根据门店key和油号 查询油价")
    public ResponseData getGasPriceDetail(@RequestParam(name = "storeKey", required = true) String storeKey,
                                          @RequestParam(name = "oilNo", required = true) String oilNo) {
        try {

            JSONObject jsonObject = TuanYouConfig.queryCompanyPriceDetail(storeKey,oilNo);
            if (jsonObject != null && jsonObject.getString("code").equals("200")) {
                if(jsonObject.getJSONArray("result").size() == 0) {
                    throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "未获取到价格信息");
                }
                return ResponseMsgUtil.success(jsonObject.get("result"));
            }
            throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "未获取到价格信息");

        } catch (Exception e) {
            log.error("HighGasController -> getGasPriceDetail() error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }

}
